/******************************************************************************
* ANAPICO LAN communication library header
* V2.2
* 21.07.2009
******************************************************************************/

#ifndef AP_LAN_LIB_H
#define AP_LAN_LIB_H

// DLL export macro
#ifdef _cplusplus
	#ifndef DLL_AP
		#define DLL_AP extern "C" __declspec(dllexport)
	#endif
#endif
#ifdef __cplusplus
	#ifndef DLL_AP
		#define DLL_AP extern "C" __declspec(dllexport)
	#endif
#endif
#ifndef DLL_AP
	#define DLL_AP
#endif

// Definition of success and error return values
#define VI_SUCCESS				0x00000000												// Operation completed successfully
#define VI_SUCCESS_MAX_CNT		0x3FFF0006												// The number of bytes read is equal to the input count
#define VI_ERROR_SYSTEM_ERROR	0xBFFF0000												// Unknown system error
#define VI_ERROR_TMO			0xBFFF0015												// Timeout expired before operation completed
#define VI_ERROR_CONN_LOST		0xBFFF00A6												// The connection for the given session has been lost
#define VI_ERROR_FILE_ACCESS	0xBFFF00A1												// An error occurred while trying to open the specified file. Possible reasons include an invalid path or lack of access rights

// APSIN communication functions
DLL_AP int ap_sock_find( char* devices, unsigned int tmo, int count, int *retCount );	// Fill string with found remote devices providing LAN control via a TCP socket
DLL_AP int ap_sock_open( char* ipAddr, int port, unsigned int tmo, int* sock_id );		// Open socket connection to remote device
DLL_AP int ap_sock_close( int sock_id );												// Close socket connection to remote device
DLL_AP int ap_sock_clear( int sock_id );												// Clear the socket buffers
DLL_AP int ap_sock_gets( int sock_id, char* buf, int count, int *retCount );			// Get a string from the remote device (control socket only)
DLL_AP int ap_sock_puts( int sock_id, char* buf, int count, int *retCount );			// Send a string to the remote device (data socket only)
DLL_AP int ap_sock_read( int sock_id, short* i16buf, int size );						// Read data from the remote device (data socket only)
DLL_AP int ap_sock_settmo( int sock_id, unsigned int tmo );								// Set timeout of socket connection, in milliseconds
DLL_AP int ap_sock_writelistfromfile( int sock_id, const char* filename );			// send a .csv LIST file to the device
DLL_AP int ap_sock_readlisttofile( int sock_id, const char* filename );			// read a .csv LIST file from the device
DLL_AP int ap_sock_sendlist( int sock_id, const char* path, const char* name );			// Send a sweep list to the remote device (control socket of APSIN devices only)
DLL_AP int ap_sock_receivelist( int sock_id, const char* path, const char* name );		// Receive a sweep list from the remote device (control socket of APSIN devices only)
DLL_AP int ap_sock_sendbinary(int sock_id, const char* path);                           // Send any file to the device
DLL_AP int ap_sock_receivebinary(int sock_id, const char* path);                        // Receive any file from the device
#endif /* AP_LAN_LIB_H */
