import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.sun.jna.Library;
import com.sun.jna.Native;

public class AnapicoUSBDemo {
	private static final int VI_SUCCESS = (int) Long.parseLong("00000000", 16);
	private static final int VI_SUCCESS_MAX_CNT = (int) Long.parseLong("3FFF0006", 16);
	private static final int VI_ERROR_SYSTEM_ERROR = (int) Long.parseLong("BFFF0000", 16);
	private static final int VI_ERROR_TMO = (int) Long.parseLong("BFFF0015", 16);
	private static final int VI_ERROR_CONN_LOST = (int) Long.parseLong("BFFF00A6", 16);
	private static final int VI_ERROR_FILE_ACCESS = (int) Long.parseLong("BFFF00A1", 16);

	public interface USBInterface extends Library {
		public int ap_usb_find(byte[] devices, int tmoMs, int count, int[] retCount);

		public int ap_usb_open(byte[] address);

		public int ap_usb_close(int linkID);

		public int ap_usb_puts(int linkID, byte[] buf, int count, int[] retCount);

		public int ap_usb_gets(int linkID, byte[] buf, int count, int[] retCount);

		public int ap_usb_read(int linkID, byte[] buf, int count, int[] retCount);

		public int ap_usb_clear(int linkID);

		public int ap_usb_settmo(int linkID, int tmoMs);
	}

	public String retToString(int ret) {
		if (ret == VI_SUCCESS) return "VI_SUCCESS";
		else if (ret == VI_SUCCESS_MAX_CNT) return "VI_SUCCESS_MAX_CNT";
		else if (ret == VI_ERROR_CONN_LOST) return "VI_ERROR_CONN_LOST";
		else if (ret == VI_ERROR_FILE_ACCESS) return "VI_ERROR_FILE_ACCESS";
		else if (ret == VI_ERROR_SYSTEM_ERROR) return "VI_ERROR_SYSTEM_ERROR";
		else if (ret == VI_ERROR_TMO) return "VI_ERROR_TMO";
		return "UNKNOWN";
	}

	private int[] retCount = new int[1];

	public AnapicoUSBDemo() {
		USBInterface lib = null;
		try {
			lib = (USBInterface) Native.loadLibrary("usb/ap_usb_lib", USBInterface.class);
		} catch (UnsatisfiedLinkError e) {
			System.out.println("Unable to load ap_usb_lib.dll.");
			return;
		}
		System.out.println("ap_usb_lib.dll loaded.");

		// search for devices
		byte[] devices = new byte[1024];
		int res = lib.ap_usb_find(devices, 200, 1024, retCount);
		System.out.printf("ap_usb_find(byte[1024], %d, %d, int[]) > %s\n", 200, 1024, retToString(res));
		if (res != VI_SUCCESS && res != VI_SUCCESS_MAX_CNT) return;
		System.out.printf(" > '%s'\n", new String(devices).trim());
		String usbAddr = getFirstFromDeviceBuffer(new String(devices).trim());
		if (usbAddr == null) return;

		// connect
		int linkID = lib.ap_usb_open(usbAddr.getBytes());
		System.out.printf("ap_usb_open('%s') > %d\n", usbAddr, linkID);
		if (linkID <= 0) return; // not connected

		// send command
		String cmd = "FREQ 100000000\n";
		res = lib.ap_usb_puts(linkID, cmd.getBytes(), cmd.getBytes().length, retCount);
		System.out.printf("ap_usb_puts('%s') > %s\n", cmd.trim(), retToString(res));
		if (res != VI_SUCCESS && res != VI_SUCCESS_MAX_CNT) return;

		// retrieve query
		cmd = "*IDN?\n";
		res = lib.ap_usb_puts(linkID, cmd.getBytes(), cmd.getBytes().length, retCount);
		System.out.printf("ap_usb_puts('%s') > %s\n", cmd.trim(), retToString(res));
		if (res != VI_SUCCESS && res != VI_SUCCESS_MAX_CNT) return;
		byte[] buf = new byte[256];
		res = lib.ap_usb_gets(linkID, buf, buf.length, retCount);
		System.out.printf("ap_usb_gets(%d) > %s\n", buf.length, retToString(res));
		String result = new String(buf).substring(0, retCount[0]).trim();
		System.out.printf(" > %d bytes: '%s'\n", retCount[0], result);
		if (res != VI_SUCCESS && res != VI_SUCCESS_MAX_CNT) return;

		// disconnect
		res = lib.ap_usb_close(linkID);
		System.out.printf("ap_usb_close() > %s\n", retToString(res));
	}

	private String getFirstFromDeviceBuffer(String deviceBuffer) {
		Matcher deviceMatcher = Pattern.compile("[0-9a-zA-Z]+:[0-9a-zA-Z-]+:[0-9a-zA-Z:-]+").matcher(deviceBuffer);
		while (deviceMatcher.find()) {
			String[] splitString = deviceMatcher.group().split(":", 3);
			String hostname = splitString[0], serial = splitString[1], usbAddress = splitString[2];
			System.out.printf(" -> extracted %s, serial:%s, usbAdress:%s\n", hostname, serial, usbAddress);
			if (hostname.contains("APPH")) return usbAddress;
		}
		return null;
	}

	public static void main(String[] args) {
		new AnapicoUSBDemo();
	}
}
